==> language-server-2.10.0-20241205-sources.jar: org/mule/weave/lsp/agent/AgentClasspathResolver.scala <==
package org.mule.weave.lsp.agent

import org.apache.commons.io.FilenameUtils
import org.mule.weave.extension.api.project.ProjectMetadata

import java.io.File
import java.io.FileFilter
import java.io.FileOutputStream
import java.nio.file.Files
import java.nio.file.Path
import java.util.Optional
import java.util.logging.Level
import java.util.logging.Logger

trait AgentClasspathResolver {
  def resolveClasspathJars(): Seq[String]
}

class SettingsBasedAgentClasspathResolver(val metadata: ProjectMetadata) extends AgentClasspathResolver {


  private lazy val resolver: AgentClasspathResolver = {
    val maybeFolder: Optional[File] = metadata.settings().agentFolder().value().map((v) => new File(v))


    if (maybeFolder.isPresent) {
      new FolderBasedAgentClasspathResolver(maybeFolder.get)
    } else {
      new ResourceBasedAgentClasspathResolver()
    }
  }

  override def resolveClasspathJars(): Seq[String] = {
    resolver.resolveClasspathJars()
  }
}

class ResourceBasedAgentClasspathResolver extends AgentClasspathResolver {
  private val LOGGER: Logger = Logger.getLogger(getClass.getName)

  private val TMP_DIR: File = new File(System.getProperty("java.io.tmpdir"))
  private val AGENT_FOLDER = "agent-server-libs"
  private val DESTINATION_FOLDER = "data-weave-agent"
  private val RESOURCES = Seq(s"$AGENT_FOLDER/agent-server.zip", s"$AGENT_FOLDER/agent-api.zip")
  private val JAR_EXTENSION = ".jar"

  lazy val maybeResolver: Option[AgentClasspathResolver] = {
    val maybeLibsDirectory = createLibsDirectory()
    maybeLibsDirectory.map(dir => new FolderBasedAgentClasspathResolver(dir))
  }

  private def createLibsDirectory(): Option[File] = {
    val url = getClass.getClassLoader.getResource(AGENT_FOLDER)
    if (url != null) {
      if (!TMP_DIR.exists()) {
        throw new RuntimeException("The specified temporary " + TMP_DIR.getAbsolutePath + " directory does not exits. Please create the directory or provide a different one.")
      }
      // Create target directory
      val targetDirectory = Files.createTempDirectory(DESTINATION_FOLDER)
      // Copy resources
      RESOURCES.foreach(resource => {
        copyJar(resource, targetDirectory)
      })
      Some(targetDirectory.toFile)
    } else {
      None
    }
  }

  private def copyJar(resource: String, targetDirectory: Path): Unit = {
    val is = getClass.getClassLoader.getResourceAsStream(resource)
    if (is != null) {
      try {
        val baseName = FilenameUtils.getBaseName(resource)
        val destination = targetDirectory.resolve(baseName + JAR_EXTENSION).toFile
        destination.createNewFile()
        destination.deleteOnExit()
        val out = new FileOutputStream(destination)
        try {
          val buffer = new Array[Byte](1024)
          Stream.continually(is.read(buffer)).takeWhile(_ != -1).foreach(out.write(buffer, 0, _))
        } finally {
          out.close()
        }
      } finally {
        is.close()
      }
    }
  }

  override def resolveClasspathJars(): Seq[String] = {
    val jars = if (maybeResolver.isDefined) {
      maybeResolver.get.resolveClasspathJars()
    } else {
      Seq.empty
    }
    if (jars.isEmpty) {
      LOGGER.log(Level.SEVERE, s"Could not resolve agent classpath jars")
    }
    jars
  }
}

class FolderBasedAgentClasspathResolver(directory: File) extends AgentClasspathResolver {
  val JAR_EXTENSION = "jar"

  lazy val jars: Seq[String] = {
    if (!directory.exists() || !directory.isDirectory) {
      Seq.empty
    } else {
      directory.listFiles(new FileFilter {
          override def accept(pathname: File): Boolean = {
            val isJar = FilenameUtils.getExtension(pathname.getName) == JAR_EXTENSION
            pathname.isFile && pathname.canRead && !pathname.isHidden && isJar
          }
        }).map(f => f.getAbsolutePath)
        .toSeq
    }
  }

  override def resolveClasspathJars(): Seq[String] = {
    jars
  }
}

==> language-server-2.10.7-20250904.153048-19-sources.jar: org/mule/weave/lsp/agent/AgentClasspathResolver.scala <==
package org.mule.weave.lsp.agent

import org.apache.commons.io.FilenameUtils
import org.mule.weave.extension.api.project.ProjectMetadata
import org.slf4j.LoggerFactory

import java.io.File
import java.io.FileFilter
import java.io.FileOutputStream
import java.nio.file.Files
import java.nio.file.Path
import java.util.Optional

trait AgentClasspathResolver {
  def resolveClasspathJars(): Seq[String]
}

class SettingsBasedAgentClasspathResolver(val metadata: ProjectMetadata) extends AgentClasspathResolver {

  private lazy val resolver: AgentClasspathResolver = {
    val maybeFolder: Optional[File] = metadata.settings().agentFolder().value().map((v) => new File(v))


    if (maybeFolder.isPresent) {
      new FolderBasedAgentClasspathResolver(maybeFolder.get)
    } else {
      new ResourceBasedAgentClasspathResolver()
    }
  }

  override def resolveClasspathJars(): Seq[String] = {
    resolver.resolveClasspathJars()
  }
}

class ResourceBasedAgentClasspathResolver extends AgentClasspathResolver {
  private val logger = LoggerFactory.getLogger(classOf[ResourceBasedAgentClasspathResolver])

  private val TMP_DIR: File = new File(System.getProperty("java.io.tmpdir"))
  private val AGENT_FOLDER = "agent-server-libs"
  private val DESTINATION_FOLDER = "data-weave-agent"
  private val RESOURCES = Seq(s"$AGENT_FOLDER/agent-server.zip", s"$AGENT_FOLDER/agent-api.zip")
  private val JAR_EXTENSION = ".jar"

  lazy val maybeResolver: Option[AgentClasspathResolver] = {
    val maybeLibsDirectory = createLibsDirectory()
    maybeLibsDirectory.map(dir => new FolderBasedAgentClasspathResolver(dir))
  }

  private def createLibsDirectory(): Option[File] = {
    val url = getClass.getClassLoader.getResource(AGENT_FOLDER)
    if (url != null) {
      if (!TMP_DIR.exists()) {
        throw new RuntimeException("The specified temporary " + TMP_DIR.getAbsolutePath + " directory does not exits. Please create the directory or provide a different one.")
      }
      // Create target directory
      val targetDirectory = Files.createTempDirectory(DESTINATION_FOLDER)
      // Copy resources
      RESOURCES.foreach(resource => {
        copyJar(resource, targetDirectory)
      })
      Some(targetDirectory.toFile)
    } else {
      None
    }
  }

  private def copyJar(resource: String, targetDirectory: Path): Unit = {
    val is = getClass.getClassLoader.getResourceAsStream(resource)
    if (is != null) {
      try {
        val baseName = FilenameUtils.getBaseName(resource)
        val destination = targetDirectory.resolve(baseName + JAR_EXTENSION).toFile
        destination.createNewFile()
        destination.deleteOnExit()
        val out = new FileOutputStream(destination)
        try {
          val buffer = new Array[Byte](1024)
          Stream.continually(is.read(buffer)).takeWhile(_ != -1).foreach(out.write(buffer, 0, _))
        } finally {
          out.close()
        }
      } finally {
        is.close()
      }
    }
  }

  override def resolveClasspathJars(): Seq[String] = {
    val jars = if (maybeResolver.isDefined) {
      maybeResolver.get.resolveClasspathJars()
    } else {
      Seq.empty
    }
    if (jars.isEmpty) {
      logger.error(s"Could not resolve agent classpath jars")
    }
    jars
  }
}

class FolderBasedAgentClasspathResolver(directory: File) extends AgentClasspathResolver {
  val JAR_EXTENSION = "jar"

  lazy val jars: Seq[String] = {
    if (!directory.exists() || !directory.isDirectory) {
      Seq.empty
    } else {
      directory.listFiles(new FileFilter {
          override def accept(pathname: File): Boolean = {
            val isJar = FilenameUtils.getExtension(pathname.getName) == JAR_EXTENSION
            pathname.isFile && pathname.canRead && !pathname.isHidden && isJar
          }
        }).map(f => f.getAbsolutePath)
        .toSeq
    }
  }

  override def resolveClasspathJars(): Seq[String] = {
    jars
  }
}
